<?php
namespace nur\data\types;

class TelephoneType extends RegexpType {
  const PATTERN = '/^[0-9+, .()-\/]+/';
  const PATTERN_NORMALIZED = '/^(\+?[0-9 ,]+)$/';

  const COD_PAYS = [
    '1242' => ['1242', null, 'Bahamas'],
    '1246' => ['1246', null, 'Barbade'],
    '1264' => ['1264', null, 'Anguilla'],
    '1268' => ['1268', null, 'Antigua-et-Barbuda'],
    '1284' => ['1284', null, 'Îles Vierges britanniques'],
    '1340' => ['1340', null, 'Îles Vierges des États-Unis'],
    '1345' => ['1345', null, 'îles Caïmans'],
    '1441' => ['1441', null, 'Bermudes'],
    '1473' => ['1473', null, 'Grenade'],
    '1649' => ['1649', null, 'Îles Turques-et-Caïques'],
    '1664' => ['1664', null, 'Montserrat'],
    '1670' => ['1670', null, 'Îles Mariannes du Nord'],
    '1671' => ['1671', null, 'Guam'],
    '1684' => ['1684', null, 'Samoa américaines'],
    '1758' => ['1758', null, 'Sainte-Lucie'],
    '1767' => ['1767', null, 'Dominique'],
    '1784' => ['1784', null, 'Saint-Vincent-et-les-Grenadines'],
    '1868' => ['1868', null, 'Trinité-et-Tobago'],
    '1869' => ['1869', null, 'Saint-Christophe-et-Niévès'],
    '1876' => ['1876', null, 'Jamaïque'],
    '1' => ['1', null, 'États-Unis'],
    ['1', null, 'Canada'],
    ['1', null, 'Porto Rico'],
    ['1', null, 'République dominicaine'],
    '20' => ['20', ['1'], 'Égypte'],
    '211' => ['211', null, 'Soudan du Sud'],
    '212' => ['212', '6' => ['6', '7'], 'Maroc'],
    '213' => ['213', '5' => ['5', '6', '7', '9'], 'Algérie'],
    '216' => ['216', '9' => ['9', '2', '5', '3', '4'], 'Tunisie'],
    '218' => ['218', null, 'Libye'],
    '220' => ['220', null, 'Gambie'],
    '221' => ['221', null, 'Sénégal'],
    '222' => ['222', null, 'Mauritanie'],
    '223' => ['223', null, 'Mali'],
    '224' => ['224', null, 'Guinée'],
    '225' => ['225', null, 'Côte d\'Ivoire'],
    '226' => ['226', null, 'Burkina Faso'],
    '227' => ['227', null, 'Niger'],
    '228' => ['228', null, 'Togo'],
    '229' => ['229', null, 'Bénin'],
    '230' => ['230', ['5'], 'Maurice'],
    '231' => ['231', null, 'Libéria Liberia'],
    '232' => ['232', null, 'Sierra Leone'],
    '233' => ['233', null, 'Ghana'],
    '234' => ['234', null, 'Nigeria'],
    '235' => ['235', null, 'Tchad'],
    '236' => ['236', null, 'République centrafricaine'],
    '237' => ['237', ['2 ou 6'], 'Cameroun'],
    '238' => ['238', null, 'Cap-Vert'],
    '239' => ['239', null, 'Sao Tomé-et-Principe'],
    '240' => ['240', null, 'Guinée équatoriale'],
    '241' => ['241', null, 'Gabon'],
    '242' => ['242', null, 'république du Congo'],
    '243' => ['243', null, 'république démocratique du Congo'],
    '244' => ['244', ['9'], 'Angola'],
    '245' => ['245', null, 'Guinée-Bissau'],
    '246' => ['246', null, 'Diego Garcia'],
    '247' => ['247', null, 'Ascension'],
    '248' => ['248', null, 'Seychelles'],
    '249' => ['249', null, 'Soudan'],
    '250' => ['250', null, 'Rwanda'],
    '251' => ['251', null, 'Éthiopie'],
    '252' => ['252', null, 'Somalie'],
    '253' => ['253', null, 'Djibouti'],
    '254' => ['254', null, 'Kenya'],
    '255' => ['255', null, 'Tanzanie'],
    '256' => ['256', null, 'Ouganda'],
    '257' => ['257', null, 'Burundi'],
    '258' => ['258', null, 'Mozambique'],
    '260' => ['260', null, 'Zambie'],
    '261' => ['261', '32' => ['32', '33', '34', '39'], 'Madagascar'],
    '262' => ['262', '692' => ['692', '693'], 'La Réunion'],
    ['262', ['639'], 'Mayotte'],
    '263' => ['263', null, 'Zimbabwe'],
    '264' => ['264', null, 'Namibie'],
    '265' => ['265', null, 'Malawi'],
    '266' => ['266', null, 'Lesotho'],
    '267' => ['267', null, 'Botswana'],
    '268' => ['268', null, 'Eswatini'],
    '269' => ['269', null, 'Comores'],
    '27' => ['27', null, 'Afrique du Sud'],
    '290' => ['290', null, 'Sainte-Hélène, Ascension et Tristan da Cunha, île'],
    '291' => ['291', null, 'Érythrée'],
    '297' => ['297', null, 'Aruba'],
    '298' => ['298', null, 'Îles Féroé'],
    '299' => ['299', null, 'Groenland'],
    '30' => ['30', ['6'], 'Grèce'],
    '31' => ['31', ['6'], 'Pays-Bas'],
    '32' => ['32', '46' => ['46', '47', '48', '49'], 'Belgique'],
    '33' => ['33', '6' => ['6', '7'], 'France'],
    '34' => ['34', '6' => ['6', '7'], 'Espagne'],
    '350' => ['350', null, 'Gibraltar'],
    '351' => ['351', ['9'], 'Portugal'],
    '352' => ['352', '621' => ['621', '661', '671', '691'], 'Luxembourg'],
    '353' => ['353', '82' => ['82', '83', '84', '85', '86', '87', '88', '89'], 'Irlande'],
    '354' => ['354', '6' => ['6', '7', '8'], 'Islande'],
    '355' => ['355', ['6'], 'Albanie'],
    '356' => ['356', null, 'Malte'],
    '357' => ['357', ['9'], 'Chypre'],
    '358' => ['358', '4' => ['4', '50'], 'Finlande'],
    '359' => ['359', '87' => ['87', '88', '89'], 'Bulgarie'],
    '36' => ['36', '20' => ['20', '30', '31', '70'], 'Hongrie'],
    '370' => ['370', ['6'], 'Lituanie'],
    '371' => ['371', ['2'], 'Lettonie'],
    '372' => ['372', '5' => ['5', '81', '82'], 'Estonie'],
    '373' => ['373', null, 'Moldavie'],
    '374' => ['374', null, 'Arménie'],
    '375' => ['375', null, 'Biélorussie'],
    '376' => ['376', '3' => ['3', '4', '6'], 'Andorre'],
    '377' => ['377', ['4'], 'Monaco'],
    '378' => ['378', null, 'Saint-Marin'],
    '380' => ['380', null, 'Ukraine'],
    '381' => ['381', '6' => ['6', '44', '45', '43', '49'], 'Serbie'],
    '382' => ['382', null, 'Monténégro'],
    '383' => ['383', null, 'Kosovo'],
    '385' => ['385', ['9'], 'Croatie'],
    '386' => ['386', '30' => ['30', '31', '40', '41', '43', '49', '51', '64', '68', '70', '71'], 'Slovénie'],
    '387' => ['387', ['6'], 'Bosnie-Herzégovine'],
    '389' => ['389', ['7'], 'Macédoine du Nord'],
    '39' => ['39', ['3'], 'Italie'],
    ['39', ['379'], 'Vatican'],
    '40' => ['40', ['7'], 'Roumanie'],
    '41' => ['41', '74' => ['74', '75', '76', '77', '78', '79'], 'Suisse'],
    '420' => ['420', null, 'Tchéquie République tchèque'],
    '421' => ['421', ['9'], 'Slovaquie'],
    '423' => ['423', null, 'Liechtenstein'],
    '43' => ['43', ['6'], 'Autriche'],
    '44' => ['44', '74' => ['74', '75', '7624', '77', '78', '79'], 'Royaume-Uni'],
    '45' => ['45', '2' => ['2', '30', '31', '40', '41', '42', '50', '51', '52', '53', '60', '61', '71', '81', '9'], 'Danemark'],
    '46' => ['46', '70' => ['70', '71', '72', '73', '76'], 'Suède'],
    '47' => ['47', '4' => ['4', '9'], 'Norvège'],
    '48' => ['48', '50' => ['50', '51', '53', '60', '66', '69', '72', '73', '78', '79', '88'], 'Pologne'],
    '49' => ['49', '15' => ['15', '16', '17'], 'Allemagne'],
    '500' => ['500', null, 'Îles Malouines'],
    '501' => ['501', null, 'Belize'],
    '502' => ['502', null, 'Guatemala'],
    '503' => ['503', null, 'Salvador'],
    '504' => ['504', null, 'Honduras'],
    '505' => ['505', null, 'Nicaragua'],
    '506' => ['506', '7' => ['7', '8'], 'Costa Rica'],
    '507' => ['507', null, 'Panama'],
    '508' => ['508', null, 'Saint-Pierre-et-Miquelon'],
    '509' => ['509', null, 'Haïti'],
    '51' => ['51', ['9'], 'Pérou'],
    '52' => ['52', ['1'], 'Mexique'],
    '53' => ['53', null, 'Cuba'],
    '54' => ['54', ['9'], 'Argentine'],
    '55' => ['55', ['xx6', 'xx7', 'xx8', 'xx9'], 'Brésil'],
    '56' => ['56', ['9'], 'Chili'],
    '57' => ['57', ['3'], 'Colombie'],
    '58' => ['58', ['4'], 'Venezuela'],
    '590' => ['590', ['690'], 'Guadeloupe'],
    '591' => ['591', '6' => ['6', '7'], 'Bolivie'],
    '592' => ['592', null, 'Guyana'],
    '593' => ['593', ['9'], 'Équateur'],
    '594' => ['594', ['694'], 'Guyane'],
    '595' => ['595', null, 'Paraguay'],
    '596' => ['596', ['696'], 'Martinique'],
    '597' => ['597', ['8'], 'Suriname'],
    '598' => ['598', null, 'Uruguay'],
    '599' => ['599', null, 'Curaçao et Pays-Bas caribéens'],
    '60' => ['60', ['1'], 'Malaisie'],
    '61' => ['61', '1' => ['1', '4'], 'Australie'],
    '62' => ['62', null, 'Indonésie'],
    '63' => ['63', null, 'Philippines'],
    '64' => ['64', ['2'], 'Nouvelle-Zélande'],
    '65' => ['65', '8' => ['8', '9'], 'Singapour'],
    '66' => ['66', ['8'], 'Thaïlande'],
    '670' => ['670', null, 'Timor oriental'],
    '672' => ['672', null, 'Christmas, Cocos, Heard-et-MacDonald, Îles'],
    '673' => ['673', null, 'Brunei'],
    '674' => ['674', null, 'Nauru'],
    '675' => ['675', null, 'Papouasie-Nouvelle-Guinée'],
    '676' => ['676', null, 'Tonga'],
    '677' => ['677', null, 'Îles Salomon'],
    '678' => ['678', null, 'Vanuatu'],
    '679' => ['679', null, 'Fidji'],
    '680' => ['680', null, 'Palaos'],
    '681' => ['681', null, 'Wallis-et-Futuna'],
    '682' => ['682', null, 'Îles Cook'],
    '683' => ['683', null, 'Niue'],
    '685' => ['685', null, 'Samoa'],
    '686' => ['686', null, 'Kiribati'],
    '687' => ['687', null, 'Nouvelle-Calédonie'],
    '688' => ['688', null, 'Tuvalu'],
    '689' => ['689', null, 'Polynésie française'],
    '690' => ['690', null, 'Tokelau'],
    '691' => ['691', null, 'États fédérés de Micronésie'],
    '692' => ['692', null, 'Îles Marshall'],
    '7' => ['7', ['9'], 'Russie'],
    ['7', '70' => ['70', '77'], 'Kazakhstan'],
    '81' => ['81', '070' => ['070', '080', '090'], 'Japon'],
    '82' => ['82', ['1'], 'Corée du Sud'],
    '84' => ['84', null, 'République socialiste du Viêt Nam'],
    '850' => ['850', null, 'Corée du Nord'],
    '852' => ['852', '5' => ['5', '6', '9'], 'Hong Kong'],
    '853' => ['853', null, 'Macao'],
    '855' => ['855', null, 'Cambodge'],
    '856' => ['856', null, 'Laos'],
    '86' => ['86', ['1'], 'République populaire de Chine'],
    '880' => ['880', null, 'Bangladesh'],
    '881' => ['881', null, 'Système mobile mondial par satellite GMSS'],
    '886' => ['886', ['9'], 'Taïwan'],
    '90' => ['90', ['5'], 'Turquie'],
    '91' => ['91', '7' => ['7', '8', '9'], 'Inde'],
    '92' => ['92', null, 'Pakistan'],
    '93' => ['93', ['7'], 'Afghanistan'],
    '94' => ['94', null, 'Sri Lanka'],
    '95' => ['95', null, 'Birmanie'],
    '960' => ['960', null, 'Maldives'],
    '961' => ['961', '3' => ['3', '70', '71'], 'Liban'],
    '962' => ['962', null, 'Jordanie'],
    '963' => ['963', null, 'Syrie'],
    '964' => ['964', null, 'Irak'],
    '965' => ['965', null, 'Koweït'],
    '966' => ['966', ['5'], 'Arabie saoudite'],
    '967' => ['967', null, 'Yémen'],
    '968' => ['968', null, 'Oman'],
    '970' => ['970', null, 'Palestine'],
    '971' => ['971', ['5'], 'Émirats arabes unis'],
    '972' => ['972', ['5'], 'Israël'],
    '973' => ['973', null, 'Bahreïn'],
    '974' => ['974', null, 'Qatar'],
    '975' => ['975', null, 'Bhoutan'],
    '976' => ['976', null, 'Mongolie'],
    '977' => ['977', null, 'Népal'],
    '98' => ['98', null, 'Iran'],
    '992' => ['992', null, 'Tadjikistan'],
    '993' => ['993', null, 'Turkménistan'],
    '994' => ['994', null, 'Azerbaïdjan'],
    '995' => ['995', null, 'Géorgie'],
    '996' => ['996', null, 'Kirghizistan'],
    '998' => ['998', null, 'Ouzbékistan'],
  ];

  private static function format_local(string $tel): string {
    return substr($tel, 0, 4)
      ." ".substr($tel, 4, 2)
      ." ".substr($tel, 6, 2)
      ." ".substr($tel, 8, 2);
  }

  private static function format_foreign(string $tel): string {
    $parts = [];
    while(strlen($tel) > 4) {
      $parts[] = substr($tel, -3);
      $tel = substr($tel, 0, -3);
    }
    $parts[] = $tel;
    $parts = array_reverse($parts);
    return implode(" ", $parts);
  }

  protected function extractParsedValue(array $ms) {
    $tel = $ms[0];
    $tel = preg_replace('/[ .()-\/]/', "", $tel);
    $tel = preg_replace('/\+/', "00", $tel);
    $ntel = preg_replace('/[^0-9]/', "", $tel);
    # tel est le numéro avec chiffres et virgule
    # ntel est le numéro avec uniquement des chiffres
    $size = strlen($ntel);
    if ($size == 4) {
      # numéro de poste, le retourner tel quel
      return $tel;
    } elseif ($size == 6) {
      # numéro local sans préfixe
      $tel = "00262262$tel";
    } elseif ($size == 10 && preg_match('/^0[1-9]/', $ntel)) {
      # numéro local avec préfixe
      $tel = "00262".substr($tel, 1);
    }
    # erreurs courantes
    if ($size == 9 && substr($ntel, 0, 1) != "0") {
      $tel = "00262$tel";
    } elseif ($size == 12 && strpos($ntel, "262262") === 0) {
      $tel = "00$tel";
    } elseif ($size == 12 && strpos($ntel, "262692") === 0) {
      $tel = "00$tel";
    } elseif ($size == 12 && strpos($ntel, "262693") === 0) {
      $tel = "00$tel";
    } elseif ($size == 12 && strpos($ntel, "000262") === 0) {
      $tel = "00262".substr($tel, 3);
    } elseif ($size == 12 && strpos($ntel, "000692") === 0) {
      $tel = "00262".substr($tel, 3);
    } elseif ($size == 12 && strpos($ntel, "000693") === 0) {
      $tel = "00262".substr($tel, 3);
    } elseif ($size == 11 && strpos($ntel, "00262") === 0) {
      $tel = "00262".substr($tel, 2);
    } elseif ($size == 11 && strpos($ntel, "00692") === 0) {
      $tel = "00262".substr($tel, 2);
    } elseif ($size == 11 && strpos($ntel, "00693") === 0) {
      $tel = "00262".substr($tel, 2);
    }
    # est-ce un numéro français?
    if (substr($tel, 0, 5) == "00262") {
      return self::format_local("0".substr($tel, 5));
    } elseif (substr($tel, 0, 4) == "0033") {
      return self::format_local("0".substr($tel, 4));
    }
    # chercher la partie internationale
    $prefix = false;
    foreach (self::COD_PAYS as $cod_pay) {
      $cod_pay = $cod_pay[0];
      $plen = strlen($cod_pay);
      if (substr($tel, 0, $plen + 2) == "00$cod_pay") {
        $prefix = "+$cod_pay";
        $tel = substr($tel, $plen + 2);
        break;
      } elseif (substr($tel, 0, $plen) == $cod_pay) {
        $prefix = "+$cod_pay";
        $tel = substr($tel, $plen);
        break;
      }
    }
    if ($prefix === false) return $tel;
    return "$prefix ".self::format_foreign($tel);
  }

  /**
   * formatter le numéro pour qu'il soit toujours au format international.
   * le numéro doit avoir déjà été formatté au préalable
   */
  function ensureInternational(?string $tel): ?string {
    if ($tel === null) return null;
    else if ($tel[0] == "+") return $tel;
    switch (substr($tel, 0, 4)) {
    case "0262":
    case "0692":
    case "0693":
      return "+262 ".substr($tel, 1);
    default:
      return "+33 ".substr($tel, 1);
    }
  }

  /**
   * formatter le numéro pour qu'il soit toujours au format local si possible.
   * c'est le "contraire" de {@link ensureInternational()}
   */
  function ensureLocal(?string $tel): ?string {
    if ($tel === null) return null;
    elseif (substr($tel, 0, 5) == "+262 ") return "0".substr($tel, 5);
    elseif (substr($tel, 0, 4) == "+33 ") return "0".substr($tel, 4);
    else return $tel;
  }

  /**
   * vérifier si $tel est un numéro de portable.
   *
   * le numéro doit avoir été formatté, que ce soit au format international ou
   * local. si ce n'est pas un numéro français, la réponse est toujours false
   */
  function isMobile(?string $tel): bool {
    if ($tel === null) return false;
    if (preg_match('/^\+(262|33) [67]/', $tel)) return true;
    if (preg_match('/^0[67]/', $tel)) return true;
    return false;
  }
}
