<?php
namespace nur\sery\db\sqlite;

use nur\sery\cl;
use nur\sery\db\CapacitorChannel;
use nur\sery\db\CapacitorStorage;
use nur\sery\php\func;
use nur\sery\ValueException;

/**
 * Class SqliteStorage
 */
class SqliteStorage extends CapacitorStorage {
  function __construct($sqlite) {
    $this->db = Sqlite::with($sqlite);
  }

  /** @var Sqlite */
  protected $db;

  function db(): Sqlite {
    return $this->db;
  }

  const PRIMARY_KEY_DEFINITION = [
    "id_" => "integer primary key autoincrement",
  ];

  protected function _create(CapacitorChannel $channel): void {
    if (!$channel->isCreated()) {
      $cols = $this->ColumnDefinitions($channel);
      $this->db->exec([
        "create table if not exists",
        "table" => $channel->getTableName(),
        "cols" => $cols,
      ]);
      $channel->setCreated();
    }
  }

  function _exists(CapacitorChannel $channel): bool {
    $tableName = $this->db->get([
      "select name from sqlite_schema",
      "where" => [
        "name" => $channel->getTableName(),
      ],
    ]);
    return $tableName !== null;
  }

  function _ensureExists(CapacitorChannel $channel): void {
    $this->_create($channel);
  }

  function _reset(CapacitorChannel $channel, bool $ensureExists=false): void {
    $this->db->exec([
      "drop table if exists",
      $channel->getTableName(),
    ]);
    $channel->setCreated(false);
    if ($ensureExists) $this->_ensureExists($channel);
  }

  function close(): void {
    $this->db->close();
  }
}
